package hal

type MultiThreadedIOPort struct {
	Module *Module
	IOPort int64
}

func (port *MultiThreadedIOPort) OutgoingIO() *MultiThreadedIO {
	if outgoingIO, ok := port.Module.IO[port.IOPort].(*MultiThreadedIO); ok {
		return outgoingIO
	}
	return nil
}

type MultiThreadedIO struct {
	channel        chan float64
	OutgoingIOPort *MultiThreadedIOPort
}

func NewMultiThreadedIO(outgoingPort *MultiThreadedIOPort) *MultiThreadedIO {
	return &MultiThreadedIO{
		channel:        make(chan float64),
		OutgoingIOPort: outgoingPort,
	}
}

func (io *MultiThreadedIO) Read() (float64, error) {
	result := <-io.OutgoingIOPort.OutgoingIO().channel
	return result, nil
}

func (io *MultiThreadedIO) Write(content float64) error {
	io.channel <- content
	return nil
}
